import { __rest } from 'tslib';
import { checkIfControllingVariants, resolveVariantFromProps } from '../../render/utils/variants.js';
import { isAnimationControls } from '../../animation/animation-controls.js';
import { useMemo, useContext } from 'react';
import { MotionContext } from '../context/MotionContext.js';
import { PresenceContext } from '../../components/AnimatePresence/PresenceContext.js';
import { resolveMotionValue } from '../../value/utils/resolve-motion-value.js';

var makeCreateVisualState = function (props, context, presenceContext, scrapeMotionValues) { return function () {
    var values = {};
    var blockInitialAnimation = (presenceContext === null || presenceContext === void 0 ? void 0 : presenceContext.initial) === false;
    var motionValues = scrapeMotionValues(props);
    for (var key in motionValues) {
        values[key] = resolveMotionValue(motionValues[key]);
    }
    var initial = props.initial, animate = props.animate;
    var isControllingVariants = checkIfControllingVariants(props);
    var isVariantNode = isControllingVariants || props.variants;
    if (context &&
        isVariantNode &&
        !isControllingVariants &&
        props.inherit !== false) {
        initial !== null && initial !== void 0 ? initial : (initial = context.initial);
        animate !== null && animate !== void 0 ? animate : (animate = context.animate);
    }
    var variantToSet = blockInitialAnimation || initial === false ? animate : initial;
    if (variantToSet &&
        typeof variantToSet !== "boolean" &&
        !isAnimationControls(variantToSet)) {
        var list = Array.isArray(variantToSet) ? variantToSet : [variantToSet];
        list.forEach(function (definition) {
            var resolved = resolveVariantFromProps(props, definition);
            if (!resolved)
                return;
            var transitionEnd = resolved.transitionEnd, transition = resolved.transition, target = __rest(resolved, ["transitionEnd", "transition"]);
            for (var key in target)
                values[key] = target[key];
            for (var key in transitionEnd)
                values[key] = transitionEnd[key];
        });
    }
    return values;
}; };
/**
 *
 */
function useCreateVisualState(props, isStatic, scrapeMotionValuesFromProps) {
    var createVisualState = makeCreateVisualState(props, useContext(MotionContext), useContext(PresenceContext), scrapeMotionValuesFromProps);
    return isStatic ? createVisualState() : useMemo(createVisualState, []);
}

export { makeCreateVisualState, useCreateVisualState };
