import { __assign } from 'tslib';
import { isSVGComponent } from './utils/is-svg-component.js';
import { createElement } from 'react';
import { useHTMLProps } from '../html/use-props.js';
import { filterProps } from './utils/filter-props.js';
import { useSVGProps } from '../svg/use-props.js';

function createUseRender(Component, forwardMotionProps) {
    if (forwardMotionProps === void 0) { forwardMotionProps = false; }
    var useRender = function (props, visualState, isStatic, visualElement) {
        var useVisualProps = isSVGComponent(Component)
            ? useSVGProps
            : useHTMLProps;
        var visualProps = useVisualProps(props, visualState, isStatic);
        var filteredProps = filterProps(props, typeof Component === "string", forwardMotionProps);
        var elementProps = __assign(__assign({}, filteredProps), visualProps);
        if (visualElement) {
            elementProps.ref = visualElement.ref;
        }
        return createElement(Component, elementProps);
    };
    return useRender;
}

export { createUseRender };
