import { __assign } from 'tslib';
import { isMotionValue } from '../../../value/utils/is-motion-value.js';
import { createLayoutState, createProjectionState } from '../../utils/state.js';
import { isForcedMotionValue } from '../../../motion/utils/is-forced-motion-value.js';
import { buildHTMLStyles } from './build-html-styles.js';
import { createHtmlRenderState } from './create-html-render-state.js';
import { useMemo } from 'react';

function useInitialMotionValues(_a, visualState, isStatic) {
    var transformTemplate = _a.transformTemplate;
    return useMemo(function () {
        var state = createHtmlRenderState();
        buildHTMLStyles(state, visualState, createProjectionState(), createLayoutState(), { enableHardwareAcceleration: !isStatic }, transformTemplate);
        var vars = state.vars, style = state.style;
        return __assign(__assign({}, vars), style);
    }, [visualState]);
}
function useStyle(props, visualState, isStatic) {
    var styleProp = props.style || {};
    var style = {};
    /**
     * Copy non-Motion Values straight into style
     */
    for (var key in styleProp) {
        // TODO We might want this to be a hasValue check? Although this could be impure
        if (!isMotionValue(styleProp[key]) &&
            !isForcedMotionValue(key, props)) {
            style[key] = styleProp[key];
        }
    }
    style = __assign(__assign({}, style), useInitialMotionValues(props, visualState, isStatic));
    if (props.transformValues) {
        style = props.transformValues(style);
    }
    return style;
}
function useHTMLProps(props, visualState, isStatic) {
    // The `any` isn't ideal but it is the type of createElement props argument
    var htmlProps = {};
    var style = useStyle(props, visualState, isStatic);
    if (Boolean(props.drag)) {
        // Disable the ghost element when a user drags
        htmlProps.draggable = false;
        // Disable text selection
        style.userSelect = style.WebkitUserSelect = style.WebkitTouchCallout =
            "none";
        // Disable scrolling on the draggable direction
        style.touchAction =
            props.drag === true
                ? "none"
                : "pan-" + (props.drag === "x" ? "y" : "x");
    }
    htmlProps.style = style;
    return htmlProps;
}

export { useHTMLProps, useStyle };
