import { __assign } from 'tslib';
import { buildSVGAttrs } from './utils/build-attrs.js';
import { createSvgRenderState } from './utils/create-render-state.js';
import { useMemo } from 'react';
import { copyRawValuesOnly } from '../html/use-props.js';

function useSVGProps(props, visualState) {
    var visualProps = useMemo(function () {
        var state = createSvgRenderState();
        buildSVGAttrs(state, visualState, undefined, undefined, { enableHardwareAcceleration: false }, props.transformTemplate);
        return __assign(__assign({}, state.attrs), { style: state.style });
    }, [visualState]);
    /**
     * Remove transform or transformOrigin - these are built relying on measurements
     * of the SVG element, which we won't have at initial render, and can't access
     * safely in concurrent mode in subsequent renders.
     */
    delete visualProps.style.transform;
    delete visualProps.style.transformOrigin;
    if (props.style) {
        var rawStyles = {};
        copyRawValuesOnly(rawStyles, props.style, props);
        visualProps.style = __assign(__assign({}, rawStyles), visualProps.style);
    }
    return visualProps;
}

export { useSVGProps };
